@@ -50,7 +50,7 @@ const hiddenSettingsDefault = {
     autoCommentFilterTemplate: '{{date}} {{origin}}',
     autoUpdateAssetFetchPeriod: 60,
     autoUpdateDelayAfterLaunch: 105,
-    autoUpdatePeriod: 4,
+    autoUpdatePeriod: 2,
     benchmarkDatasetURL: 'unset',
     blockingProfiles: '11111/#F00 11010/#C0F 11001/#00F 00001',
     cacheStorageAPI: 'unset',
@@ -63,9 +63,9 @@ const hiddenSettingsDefault = {
     cnameIgnoreRootDocument: true,
     cnameMaxTTL: 120,
     cnameReplayFullURL: false,
-    cnameUncloak: true,
     cnameUncloakProxied: false,
     consoleLogLevel: 'unset',
+    debugAssetsJson: false,
     debugScriptlets: false,
     debugScriptletInjector: false,
     disableWebAssembly: false,
@@ -82,6 +82,7 @@ const hiddenSettingsDefault = {
     requestJournalProcessPeriod: 1000,
     selfieAfter: 2,
     strictBlockingBypassDuration: 120,
+    toolbarWarningTimeout: 60,
     uiPopupConfig: 'unset',
     uiStyles: 'unset',
     updateAssetBypassBrowserCache: false,
@@ -172,7 +173,6 @@ const µBlock = {  // jshint ignore:line
         allowedRequestCount: 0,
     },
     localSettingsLastModified: 0,
-    localSettingsLastSaved: 0,
 
     // Read-only
     systemSettings: {
@@ -219,8 +219,8 @@ const µBlock = {  // jshint ignore:line
     readyToFilter: false,
 
     supportStats: {
-        allReadyAfter: '',
-        maxAssetCacheWait: '0 ms',
+        allReadyAfter: '?',
+        maxAssetCacheWait: '?',
     },
 
     pageStores: new Map(),
