@@ -19,13 +19,10 @@
     Home: https://github.com/gorhill/uBlock
 */
 
-/* global uDom */
-
 'use strict';
 
-/******************************************************************************/
-
 import { i18n$ } from './i18n.js';
+import { dom, qs$ } from './dom.js';
 
 /******************************************************************************/
 
@@ -56,29 +53,32 @@ let details = {};
         }
     }
 
-    if ( Array.isArray(lists) === false || lists.length === 0 ) { return; }
+    if ( Array.isArray(lists) === false || lists.length === 0 ) {
+        qs$('#whyex').style.setProperty('visibility', 'collapse');
+        return;
+    }
 
-    const parent = uDom.nodeFromSelector('#whyex > ul');
+    const parent = qs$('#whyex > ul');
+    parent.firstElementChild.remove(); // remove placeholder element
     for ( const list of lists ) {
-        const listElem = document.querySelector('#templates .filterList')
-                                 .cloneNode(true);
-        const sourceElem = listElem.querySelector('.filterListSource');
+        const listElem = dom.clone('#templates .filterList');
+        const sourceElem = qs$(listElem, '.filterListSource');
         sourceElem.href += encodeURIComponent(list.assetKey);
-        sourceElem.textContent = list.title;
+        dom.text(sourceElem, list.title);
         if ( typeof list.supportURL === 'string' && list.supportURL !== '' ) {
-            const supportElem = listElem.querySelector('.filterListSupport');
-            supportElem.setAttribute('href', list.supportURL);
-            supportElem.classList.remove('hidden');
+            const supportElem = qs$(listElem, '.filterListSupport');
+            dom.attr(supportElem, 'href', list.supportURL);
+            dom.cl.remove(supportElem, 'hidden');
         }
         parent.appendChild(listElem);
     }
-    uDom.nodeFromId('whyex').style.removeProperty('display');
+    qs$('#whyex').style.removeProperty('visibility');
 })();
 
 /******************************************************************************/
 
-uDom.nodeFromSelector('#theURL > p > span:first-of-type').textContent = details.url;
-uDom.nodeFromId('why').textContent = details.fs;
+dom.text('#theURL > p > span:first-of-type', details.url);
+dom.text('#why', details.fs);
 
 /******************************************************************************/
 
@@ -95,20 +95,21 @@ uDom.nodeFromId('why').textContent = details.fs;
             value = name;
             name = '';
         }
-        const li = document.createElement('li');
-        let span = document.createElement('span');
-        span.textContent = name;
+        const li = dom.create('li');
+        let span = dom.create('span');
+        dom.text(span, name);
         li.appendChild(span);
         if ( name !== '' && value !== '' ) {
             li.appendChild(document.createTextNode(' = '));
         }
-        span = document.createElement('span');
+        span = dom.create('span');
         if ( reURL.test(value) ) {
-            const a = document.createElement('a');
-            a.href = a.textContent = value;
+            const a = dom.create('a');
+            dom.attr(a, 'href', value);
+            dom.text(a, value);
             span.appendChild(a);
         } else {
-            span.textContent = value;
+            dom.text(span, value);
         }
         li.appendChild(span);
         return li;
@@ -135,7 +136,7 @@ uDom.nodeFromId('why').textContent = details.fs;
         for ( const [ name, value ] of params ) {
             const li = liFromParam(name, value);
             if ( depth < 2 && reURL.test(value) ) {
-                const ul = document.createElement('ul');
+                const ul = dom.create('ul');
                 renderParams(ul, value, depth + 1);
                 li.appendChild(ul);
             }
@@ -145,27 +146,22 @@ uDom.nodeFromId('why').textContent = details.fs;
         return true;
     };
 
-    if ( renderParams(uDom.nodeFromId('parsed'), details.url) === false ) {
+    if ( renderParams(qs$('#parsed'), details.url) === false ) {
         return;
     }
 
-    const toggler = document.querySelector('#toggleParse');
-    toggler.classList.remove('hidden');
+    dom.cl.remove('#toggleParse', 'hidden');
 
-    toggler.addEventListener('click', ( ) => {
-        const cl = uDom.nodeFromId('theURL').classList;
-        cl.toggle('collapsed');
+    dom.on('#toggleParse', 'click', ( ) => {
+        dom.cl.toggle('#theURL', 'collapsed');
         vAPI.localStorage.setItem(
             'document-blocked-expand-url',
-            (cl.contains('collapsed') === false).toString()
+            (dom.cl.has('#theURL', 'collapsed') === false).toString()
         );
     });
 
     vAPI.localStorage.getItemAsync('document-blocked-expand-url').then(value => {
-        uDom.nodeFromId('theURL').classList.toggle(
-            'collapsed',
-            value !== 'true' && value !== true
-        );
+        dom.cl.toggle('#theURL', 'collapsed', value !== 'true' && value !== true);
     });
 })();
 
@@ -174,23 +170,17 @@ uDom.nodeFromId('why').textContent = details.fs;
 // https://www.reddit.com/r/uBlockOrigin/comments/breeux/close_this_window_doesnt_work_on_firefox/
 
 if ( window.history.length > 1 ) {
-    uDom('#back').on(
-        'click',
-        ( ) => {
-            window.history.back();
-        }
-    );
-    uDom('#bye').css('display', 'none');
+    dom.on('#back', 'click', ( ) => {
+        window.history.back();
+    });
+    qs$('#bye').style.display = 'none';
 } else {
-    uDom('#bye').on(
-        'click',
-        ( ) => {
-            messaging.send('documentBlocked', {
-                what: 'closeThisTab',
-            });
-        }
-    );
-    uDom('#back').css('display', 'none');
+    dom.on('#bye', 'click', ( ) => {
+        messaging.send('documentBlocked', {
+            what: 'closeThisTab',
+        });
+    });
+    qs$('#back').style.display = 'none';
 }
 
 /******************************************************************************/
@@ -223,15 +213,14 @@ const proceedPermanent = async function() {
     proceedToURL();
 };
 
-uDom('#disableWarning').on('change', ev => {
+dom.on('#disableWarning', 'change', ev => {
     const checked = ev.target.checked;
-    document.querySelector('[data-i18n="docblockedBack"]').classList.toggle('disabled', checked);
-    document.querySelector('[data-i18n="docblockedClose"]').classList.toggle('disabled', checked);
+    dom.cl.toggle('[data-i18n="docblockedBack"]', 'disabled', checked);
+    dom.cl.toggle('[data-i18n="docblockedClose"]', 'disabled', checked);
 });
 
-uDom('#proceed').on('click', ( ) => {
-    const input = document.querySelector('#disableWarning');
-    if ( input.checked ) {
+dom.on('#proceed', 'click', ( ) => {
+    if ( qs$('#disableWarning').checked ) {
         proceedPermanent();
     } else {
         proceedTemporary();
