@@ -28,7 +28,9 @@ import logger from './logger.js';
 import scriptletFilteringEngine from './scriptlet-filtering.js';
 import staticNetFilteringEngine from './static-net-filtering.js';
 import µb from './background.js';
+import webext from './webext.js';
 import { PageStore } from './pagestore.js';
+import { i18n$ } from './i18n.js';
 
 import {
     sessionFirewall,
@@ -39,7 +41,6 @@ import {
 import {
     domainFromHostname,
     hostnameFromURI,
-    isNetworkURI,
     originFromURI,
 } from './uri-utils.js';
 
@@ -62,7 +63,11 @@ import {
         if ( tabId < 0 ) {
             return 'http://behind-the-scene/';
         }
-        tabURLNormalizer.href = tabURL;
+        try {
+            tabURLNormalizer.href = tabURL;
+        } catch(ex) {
+            return tabURL;
+        }
         const protocol = tabURLNormalizer.protocol.slice(0, -1);
         if ( protocol === 'https' || protocol === 'http' ) {
             return tabURLNormalizer.href;
@@ -311,9 +316,24 @@ const onPopupUpdated = (( ) => {
         if ( tabContext === null ) { return; }
         const rootOpenerURL = tabContext.rawURL;
         if ( rootOpenerURL === '' ) { return; }
-        const localOpenerURL = openerDetails.frameId !== 0
+        const pageStore = µb.pageStoreFromTabId(openerTabId);
+
+        // https://github.com/uBlockOrigin/uBlock-issues/discussions/2534#discussioncomment-5264792
+        //   An `about:blank` frame's context is that of the parent context
+        let localOpenerURL = openerDetails.frameId !== 0
             ? openerDetails.frameURL
             : undefined;
+        if ( localOpenerURL === 'about:blank' && pageStore !== null ) {
+            let openerFrameId = openerDetails.frameId;
+            do {
+                const frame = pageStore.getFrameStore(openerFrameId);
+                if ( frame === null ) { break; }
+                openerFrameId = frame.parentId;
+                const parentFrame = pageStore.getFrameStore(openerFrameId);
+                if ( parentFrame === null ) { break; }
+                localOpenerURL = parentFrame.frameURL;
+            } while ( localOpenerURL === 'about:blank' && openerFrameId !== 0 );
+        }
 
         // Popup details.
         tabContext = µb.tabContextManager.lookup(targetTabId);
@@ -334,14 +354,9 @@ const onPopupUpdated = (( ) => {
             return;
         }
 
-        // If the page URL is that of our "blocked page" URL, extract the URL
+        // If the page URL is that of our document-blocked URL, extract the URL
         // of the page which was blocked.
-        if ( targetURL.startsWith(vAPI.getURL('document-blocked.html')) ) {
-            const matches = /details=([^&]+)/.exec(targetURL);
-            if ( matches !== null ) {
-                targetURL = JSON.parse(decodeURIComponent(matches[1])).url;
-            }
-        }
+        targetURL = µb.pageURLFromMaybeDocumentBlockedURL(targetURL);
 
         // MUST be reset before code below is called.
         const fctxt = µb.filteringContext.duplicate();
@@ -350,8 +365,14 @@ const onPopupUpdated = (( ) => {
         let popupType = 'popup',
             result = 0;
         // https://github.com/gorhill/uBlock/issues/2919
-        // - If the target tab matches a clicked link, assume it's legit.
-        if ( areDifferentURLs(targetURL, openerDetails.trustedURL) ) {
+        //   If the target tab matches a clicked link, assume it's legit.
+        // https://github.com/uBlockOrigin/uBlock-issues/issues/1912
+        //   If the target also matches the last clicked link, assume it's
+        //   legit.
+        if (
+            areDifferentURLs(targetURL, openerDetails.trustedURL) &&
+            areDifferentURLs(targetURL, µb.maybeGoodPopup.url)
+        ) {
             result = popupMatch(fctxt, rootOpenerURL, localOpenerURL, targetURL);
         }
 
@@ -386,7 +407,6 @@ const onPopupUpdated = (( ) => {
 
         // Only if a popup was blocked do we report it in the dynamic
         // filtering pane.
-        const pageStore = µb.pageStoreFromTabId(openerTabId);
         if ( pageStore ) {
             pageStore.journalAddRequest(fctxt, result);
             pageStore.popupBlockedCount += 1;
@@ -495,25 +515,19 @@ housekeep itself.
                     ? µb.maybeGoodPopup.url
                     : ''
             };
-            this.selfDestructionTimer = null;
+            this.selfDestructionTimer = vAPI.defer.create(( ) => {
+                this.destroy();
+            });
             this.launchSelfDestruction();
         }
 
         destroy() {
-            if ( this.selfDestructionTimer !== null ) {
-                clearTimeout(this.selfDestructionTimer);
-            }
+            this.selfDestructionTimer.off();
             popupCandidates.delete(this.targetTabId);
         }
 
         launchSelfDestruction() {
-            if ( this.selfDestructionTimer !== null ) {
-                clearTimeout(this.selfDestructionTimer);
-            }
-            this.selfDestructionTimer = vAPI.setTimeout(
-                ( ) => this.destroy(),
-                10000
-            );
+            this.selfDestructionTimer.offon(10000);
         }
     };
 
@@ -598,8 +612,12 @@ housekeep itself.
         this.origin =
         this.rootHostname =
         this.rootDomain = '';
-        this.commitTimer = null;
-        this.gcTimer = null;
+        this.commitTimer = vAPI.defer.create(( ) => {
+            this.onCommit();
+        });
+        this.gcTimer = vAPI.defer.create(( ) => {
+            this.onGC();
+        });
         this.onGCBarrier = false;
         this.netFiltering = true;
         this.netFilteringReadTime = 0;
@@ -609,28 +627,20 @@ housekeep itself.
 
     TabContext.prototype.destroy = function() {
         if ( vAPI.isBehindTheSceneTabId(this.tabId) ) { return; }
-        if ( this.gcTimer !== null ) {
-            clearTimeout(this.gcTimer);
-            this.gcTimer = null;
-        }
+        this.gcTimer.off();
         tabContexts.delete(this.tabId);
     };
 
     TabContext.prototype.onGC = async function() {
         if ( vAPI.isBehindTheSceneTabId(this.tabId) ) { return; }
-        // https://github.com/gorhill/uBlock/issues/1713
-        //   For unknown reasons, Firefox's setTimeout() will sometimes
-        //   causes the callback function to be called immediately, bypassing
-        //   the main event loop. For now this should prevent uBO from
-        //   crashing as a result of the bad setTimeout() behavior.
         if ( this.onGCBarrier ) { return; }
         this.onGCBarrier = true;
-        this.gcTimer = null;
+        this.gcTimer.off();
         const tab = await vAPI.tabs.get(this.tabId);
         if ( tab instanceof Object === false || tab.discarded === true ) {
             this.destroy();
         } else {
-            this.gcTimer = vAPI.setTimeout(( ) => this.onGC(), gcPeriod);
+            this.gcTimer.on(gcPeriod);
         }
         this.onGCBarrier = false;
     };
@@ -639,10 +649,8 @@ housekeep itself.
     // Stack entries have to be committed to stick. Non-committed stack
     // entries are removed after a set delay.
     TabContext.prototype.onCommit = function() {
-        if ( vAPI.isBehindTheSceneTabId(this.tabId) ) {
-            return;
-        }
-        this.commitTimer = null;
+        if ( vAPI.isBehindTheSceneTabId(this.tabId) ) { return; }
+        this.commitTimer.off();
         // Remove uncommitted entries at the top of the stack.
         let i = this.stack.length;
         while ( i-- ) {
@@ -667,11 +675,14 @@ housekeep itself.
     // want to flush it.
     TabContext.prototype.autodestroy = function() {
         if ( vAPI.isBehindTheSceneTabId(this.tabId) ) { return; }
-        this.gcTimer = vAPI.setTimeout(( ) => this.onGC(), gcPeriod);
+        this.gcTimer.on(gcPeriod);
     };
 
     // Update just force all properties to be updated to match the most recent
     // root URL.
+    // https://github.com/uBlockOrigin/uBlock-issues/issues/1954
+    //   In case of document-blocked page, use the blocked page URL as the
+    //   context.
     TabContext.prototype.update = function() {
         this.netFilteringReadTime = 0;
         if ( this.stack.length === 0 ) {
@@ -683,7 +694,7 @@ housekeep itself.
             return;
         }
         const stackEntry = this.stack[this.stack.length - 1];
-        this.rawURL = stackEntry.url;
+        this.rawURL = µb.pageURLFromMaybeDocumentBlockedURL(stackEntry.url);
         this.normalURL = µb.normalizeTabURL(this.tabId, this.rawURL);
         this.origin = originFromURI(this.normalURL);
         this.rootHostname = hostnameFromURI(this.origin);
@@ -704,10 +715,7 @@ housekeep itself.
         this.stack.push(new StackEntry(url));
         this.update();
         popupCandidateTest(this.tabId);
-        if ( this.commitTimer !== null ) {
-            clearTimeout(this.commitTimer);
-        }
-        this.commitTimer = vAPI.setTimeout(( ) => this.onCommit(), 500);
+        this.commitTimer.offon(500);
     };
 
     // This tells that the url is definitely the one to be associated with the
@@ -868,11 +876,18 @@ housekeep itself.
 
 vAPI.Tabs = class extends vAPI.Tabs {
     onActivated(details) {
+        const { tabId } = details;
+        if ( vAPI.isBehindTheSceneTabId(tabId) ) { return; }
+        // https://github.com/uBlockOrigin/uBlock-issues/issues/757
+        const pageStore = µb.pageStoreFromTabId(tabId);
+        if ( pageStore === null ) {
+            this.onNewTab(tabId);
+            return;
+        }
         super.onActivated(details);
-        if ( vAPI.isBehindTheSceneTabId(details.tabId) ) { return; }
         // https://github.com/uBlockOrigin/uBlock-issues/issues/680
-        µb.updateToolbarIcon(details.tabId);
-        contextMenu.update(details.tabId);
+        µb.updateToolbarIcon(tabId);
+        contextMenu.update(tabId);
     }
 
     onClosed(tabId) {
@@ -900,6 +915,10 @@ vAPI.Tabs = class extends vAPI.Tabs {
     //   For non-network URIs, defer scriptlet injection to content script. The
     //   reason for this is that we need the effective URL and this information
     //   is not available at this point.
+    //
+    // https://github.com/uBlockOrigin/uBlock-issues/issues/2343
+    //   uBO's isolated world in Firefox just does not work as expected at
+    //   point, so we have to wait before injecting scriptlets.
     onNavigation(details) {
         super.onNavigation(details);
         const { frameId, tabId, url } = details;
@@ -913,19 +932,24 @@ vAPI.Tabs = class extends vAPI.Tabs {
         const pageStore = µb.pageStoreFromTabId(tabId);
         if ( pageStore === null ) { return; }
         pageStore.setFrameURL(details);
-        if (
-            µb.canInjectScriptletsNow &&
-            isNetworkURI(url) &&
-            pageStore.getNetFilteringSwitch()
-        ) {
+        if ( pageStore.getNetFilteringSwitch() ) {
             scriptletFilteringEngine.injectNow(details);
         }
     }
 
+    async onNewTab(tabId) {
+        const tab = await vAPI.tabs.get(tabId);
+        if ( tab === null ) { return; }
+        const { id, url = '' } = tab;
+        if ( url === '' ) { return; }
+        µb.tabContextManager.commit(id, url);
+        µb.bindTabToPageStore(id, 'tabUpdated', tab);
+        contextMenu.update(id);
+    }
+
     // It may happen the URL in the tab changes, while the page's document
     // stays the same (for instance, Google Maps). Without this listener,
     // the extension icon won't be properly refreshed.
-
     onUpdated(tabId, changeInfo, tab) {
         super.onUpdated(tabId, changeInfo, tab);
         if ( !tab.url || tab.url === '' ) { return; }
@@ -1027,14 +1051,14 @@ vAPI.tabs = new vAPI.Tabs();
     };
     const pageStore = new NoPageStore(vAPI.noTabId);
     µb.pageStores.set(pageStore.tabId, pageStore);
-    pageStore.title = vAPI.i18n('logBehindTheScene');
+    pageStore.title = i18n$('logBehindTheScene');
 }
 
 /******************************************************************************/
 
 // Update visual of extension icon.
 
-µb.updateToolbarIcon = (( ) => {
+{
     const tabIdToDetails = new Map();
 
     const computeBadgeColor = (bits) => {
@@ -1093,7 +1117,8 @@ vAPI.tabs = new vAPI.Tabs();
     //        bit 2 = badge color
     //        bit 3 = hide badge
 
-    return function(tabId, newParts = 0b0111) {
+    µb.updateToolbarIcon = function(tabId, newParts = 0b0111) {
+        if ( this.readyToFilter === false ) { return; }
         if ( typeof tabId !== 'number' ) { return; }
         if ( vAPI.isBehindTheSceneTabId(tabId) ) { return; }
         const currentParts = tabIdToDetails.get(tabId);
@@ -1108,7 +1133,7 @@ vAPI.tabs = new vAPI.Tabs();
         }
         tabIdToDetails.set(tabId, newParts);
     };
-})();
+}
 
 /******************************************************************************/
 
@@ -1116,7 +1141,6 @@ vAPI.tabs = new vAPI.Tabs();
 //   Stale page store entries janitor
 
 {
-    const pageStoreJanitorPeriod = 15 * 60 * 1000;
     let pageStoreJanitorSampleAt = 0;
     let pageStoreJanitorSampleSize = 10;
 
@@ -1142,10 +1166,13 @@ vAPI.tabs = new vAPI.Tabs();
         }
         pageStoreJanitorSampleAt = n;
 
-        vAPI.setTimeout(pageStoreJanitor, pageStoreJanitorPeriod);
+        pageStoreJanitorTimer.on(pageStoreJanitorPeriod);
     };
 
-    vAPI.setTimeout(pageStoreJanitor, pageStoreJanitorPeriod);
+    const pageStoreJanitorTimer = vAPI.defer.create(pageStoreJanitor);
+    const pageStoreJanitorPeriod = { min: 15 };
+
+    pageStoreJanitorTimer.on(pageStoreJanitorPeriod);
 }
 
 /******************************************************************************/
